@@ -80,6 +80,8 @@ class Cell : public MWAWCell
 public:
   /// constructor
   Cell() : m_content(), m_style(-1) { }
+  /// destructor
+  virtual ~Cell();
   //! returns true if the cell do contain any content
   bool isEmpty() const
   {
@@ -92,6 +94,10 @@ public:
   int m_style;
 };
 
+Cell::~Cell()
+{
+}
+
 /** the spreadsheet of a of a MsWksSSParser */
 class Spreadsheet
 {
@@ -104,10 +110,27 @@ public:
   //! returns the row size in point
   int getRowHeight(int row) const
   {
-    if (row>=0&&row<(int) m_heightRows.size())
+    if (row>=0&&row<static_cast<int>(m_heightRows.size()))
       return m_heightRows[size_t(row)];
     return m_heightDefault;
   }
+  //! returns the height of a row in point and updated repeated row
+  double getRowHeight(int row, int &numRepeated) const
+  {
+    double res=getRowHeight(row);
+    numRepeated=1;
+    if (row<0 || row>=static_cast<int>(m_heightRows.size()))
+      numRepeated=1000;
+    else {
+      for (size_t r=size_t(row+1); r<m_heightRows.size(); ++r) {
+        double nextH=getRowHeight(row+1);
+        if (nextH<res || nextH>res)
+          break;
+        ++numRepeated;
+      }
+    }
+    return res;
+  }
   //! convert the m_widthCols in a vector of of point size
   std::vector<float> convertInPoint(std::vector<int> const &list) const
   {
@@ -134,16 +157,16 @@ public:
   std::string m_name;
 protected:
   /** returns the last Right Bottom cell position */
-  Vec2i getRightBottomPosition() const
+  MWAWVec2i getRightBottomPosition() const
   {
     int maxX = 0, maxY = 0;
     size_t numCell = m_cells.size();
     for (size_t i = 0; i < numCell; i++) {
-      Vec2i const &p = m_cells[i].position();
+      MWAWVec2i const &p = m_cells[i].position();
       if (p[0] > maxX) maxX = p[0];
       if (p[1] > maxY) maxY = p[1];
     }
-    return Vec2i(maxX, maxY);
+    return MWAWVec2i(maxX, maxY);
   }
 };
 
@@ -213,11 +236,13 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type
     MWAW_DEBUG_MSG(("GreatWksSSParserInternal::SubDocument::parse: unknown type\n"));
     return;
   }
-
-  assert(m_parser);
-
+  GreatWksSSParser *parser=dynamic_cast<GreatWksSSParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("GreatWksSSParserInternal::SubDocument::parse: no parser\n"));
+    return;
+  }
   long pos = m_input->tell();
-  static_cast<GreatWksSSParser *>(m_parser)->sendHF(m_id);
+  parser->sendHF(m_id);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
 
@@ -270,8 +295,7 @@ bool GreatWksSSParser::sendHF(int id)
 ////////////////////////////////////////////////////////////
 void GreatWksSSParser::parse(librevenge::RVNGSpreadsheetInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
     // create the asciiFile
@@ -382,7 +406,7 @@ bool GreatWksSSParser::readSpreadsheet()
     bool ok=true, printDone=false;
     long pos = input->tell();
     f.str("");
-    int type=(int) input->readLong(2);
+    int type=static_cast<int>(input->readLong(2));
     if (type==0x10) {
       if (vers==1)
         f << "Entries(DocInfo):";
@@ -422,19 +446,19 @@ bool GreatWksSSParser::readSpreadsheet()
       break;
     case 3: // column
     case 5: { // row :
-      long sz=(long) input->readULong(2);
-      endPos=pos+4+(long) sz;
+      long sz=long(input->readULong(2));
+      endPos=pos+4+long(sz);
       ok = (sz>=4 && input->checkPosition(endPos));
       if (!ok) break;
-      int id=(int) input->readLong(2);
-      int length=(int) input->readLong(2);
+      int id=static_cast<int>(input->readLong(2));
+      int length=static_cast<int>(input->readLong(2));
       if (id<1 || length < 0) {
         MWAW_DEBUG_MSG(("GreatWksSSParser::readSpreadsheet: the row/col definition seems bad\n"));
         f << "###";
       }
       else {
         std::vector<int> &lengths=type==3 ? m_state->m_spreadsheet.m_widthCols : m_state->m_spreadsheet.m_heightRows;
-        if (id-1 >= (int) lengths.size())
+        if (id-1 >= static_cast<int>(lengths.size()))
           lengths.resize(size_t(id), -1);
         lengths[size_t(id-1)]=length;
       }
@@ -454,7 +478,7 @@ bool GreatWksSSParser::readSpreadsheet()
         break;
       }
       for (int i=0; i < 4; ++i) {
-        int val=(int) input->readLong(2);
+        int val=static_cast<int>(input->readLong(2));
         static int const expected[]= {9,0,2,0x2e};
         if (val!=expected[i])
           f << "f" << i << "=" << val << ",";
@@ -468,7 +492,7 @@ bool GreatWksSSParser::readSpreadsheet()
     case 0xc: // d
     case 0x11: // 0
     case 0x1a: { // 1
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -479,7 +503,7 @@ bool GreatWksSSParser::readSpreadsheet()
         MWAW_DEBUG_MSG(("GreatWksSSParser::readSpreadsheet: size of zone %d seems odd\n", type));
         break;
       }
-      int val=(int)input->readULong(2);
+      int val=static_cast<int>(input->readULong(2));
       switch (type) {
       case 7:
         m_state->m_spreadsheet.m_heightDefault=val;
@@ -503,7 +527,7 @@ bool GreatWksSSParser::readSpreadsheet()
       break;
     }
     case 0xd: {
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -516,17 +540,17 @@ bool GreatWksSSParser::readSpreadsheet()
       }
       int dim[4];
       for (int i=0; i<4; ++i)
-        dim[i]=(int)input->readLong(2);
+        dim[i]=static_cast<int>(input->readLong(2));
       f << dim[0] << "x" << dim[1] << "<->" << dim[2] << "x" << dim[3] << ",";
       if (vers==1) break;
       for (int i=0; i<4; ++i)
-        dim[i]=(int)input->readLong(2);
+        dim[i]=static_cast<int>(input->readLong(2));
       f << "screen2?=" << dim[0] << "x" << dim[1] << "<->" << dim[1] << "x" << dim[2] << ",";
       // now find 00000003cccc3333
       break;
     }
     case 0xe: {
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -539,7 +563,7 @@ bool GreatWksSSParser::readSpreadsheet()
       }
       int val;
       for (int i=0; i<9; ++i) {
-        val=(int)input->readULong(1);
+        val=static_cast<int>(input->readULong(1));
         if (!val) continue;
         // checkme: same value in v2 ?
         static char const *(what[])= {"grid[display]", "headerRC[display]", "zero[doc]", "formula[doc]", "grid[print]", "headerRC[print]", "header[print]", "footer[print]", "protected" };
@@ -553,14 +577,14 @@ bool GreatWksSSParser::readSpreadsheet()
         else
           f << "###" << what[i] << "=" << val << ",";
       }
-      val=(int)input->readULong(1);
+      val=static_cast<int>(input->readULong(1));
       if (val!=1) f << "first[page]=" << val << ",";
       break;
     }
     case 0xf: {
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       endPos=pos+6+sz;
-      if (!input->checkPosition(endPos)) {
+      if (sz<0 || !input->checkPosition(endPos)) {
         ok=false;
         break;
       }
@@ -569,7 +593,7 @@ bool GreatWksSSParser::readSpreadsheet()
         MWAW_DEBUG_MSG(("GreatWksSSParser::readSpreadsheet: size of selection zone seems odd\n"));
         break;
       }
-      int val=(int)input->readULong(2);
+      int val=static_cast<int>(input->readULong(2));
       if (val != 4) f << "f0=" << val << ",";
       f << "cells=[";
       for (int i=0; i<3; ++i) // probably selected cell, followed by a cell range
@@ -578,7 +602,7 @@ bool GreatWksSSParser::readSpreadsheet()
       break;
     }
     case 0x10: {
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -592,12 +616,12 @@ bool GreatWksSSParser::readSpreadsheet()
       }
       int dim[4];
       for (int i=0; i<4; ++i)
-        dim[i]=(int)input->readLong(2);
+        dim[i]=static_cast<int>(input->readLong(2));
       f << "page=" << dim[0] << "x" << dim[1] << "<->" << dim[2] << "x" << dim[3] << ",";
       if (input->readLong(1)!=1) break;
       input->seek(1,librevenge::RVNG_SEEK_CUR);
       for (int i=0; i<4; ++i)
-        dim[i]=(int)input->readLong(2);
+        dim[i]=static_cast<int>(input->readLong(2));
       f << "select=" << dim[0] << "x" << dim[1] << "<->" << dim[2] << "x" << dim[3] << ",";
       break;
     }
@@ -618,7 +642,7 @@ bool GreatWksSSParser::readSpreadsheet()
       ascii().addNote("_");
       return true;
     case 0x15: {
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -629,14 +653,14 @@ bool GreatWksSSParser::readSpreadsheet()
         MWAW_DEBUG_MSG(("GreatWksSSParser::readSpreadsheet: size of grid option zone seems odd\n"));
         break;
       }
-      int val=(int)input->readLong(2);
+      int val=static_cast<int>(input->readLong(2));
       if (val!=1) f << "unit=" << val << ",";
-      val=(int)input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val!=5) f << "subdiv=" << val << ",";
       break;
     }
     case 0x16: {
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -655,7 +679,7 @@ bool GreatWksSSParser::readSpreadsheet()
         // two times the same values: related to header/footer ?
         f << "unkn" << c << "=[";
         for (int j=0; j<8; ++j) {
-          int val=(int)input->readLong(2);
+          int val=static_cast<int>(input->readLong(2));
           if (val)
             f << val << ",";
           else
@@ -667,7 +691,7 @@ bool GreatWksSSParser::readSpreadsheet()
     }
     case 0x17:
     case 0x18: {
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -700,7 +724,7 @@ bool GreatWksSSParser::readSpreadsheet()
       endPos=input->tell();
       break;
     default: {
-      endPos=pos+6+(long) input->readULong(4);
+      endPos=pos+6+long(input->readULong(4));
       if (type <= 0 || !input->checkPosition(endPos)) {
         ok=false;
         break;
@@ -711,7 +735,7 @@ bool GreatWksSSParser::readSpreadsheet()
     if (!ok && type>=7) {
       // try to use the default value
       input->seek(pos+2, librevenge::RVNG_SEEK_SET);
-      long sz= (long) input->readULong(4);
+      long sz= long(input->readULong(4));
       endPos=pos+6+sz;
       ok=input->checkPosition(endPos);
     }
@@ -743,20 +767,27 @@ bool GreatWksSSParser::sendSpreadsheet()
   GreatWksSSParserInternal::Spreadsheet &sheet = m_state->m_spreadsheet;
   size_t numCell = sheet.m_cells.size();
 
-  listener->openSheet(sheet.convertInPoint(sheet.m_widthCols), librevenge::RVNG_POINT, sheet.m_name);
+  listener->openSheet(sheet.convertInPoint(sheet.m_widthCols), librevenge::RVNG_POINT, std::vector<int>(), sheet.m_name);
   m_document->getGraphParser()->sendPageGraphics();
 
   int prevRow = -1;
   for (size_t i = 0; i < numCell; i++) {
     GreatWksSSParserInternal::Cell cell= sheet.m_cells[i];
-    if (cell.position()[1] != prevRow) {
-      while (cell.position()[1] > prevRow) {
-        if (prevRow != -1)
-          listener->closeSheetRow();
-        prevRow++;
-        listener->openSheetRow((float)sheet.getRowHeight(prevRow), librevenge::RVNG_POINT);
+    if (cell.position()[1]>prevRow+1) {
+      while (cell.position()[1] > prevRow+1) {
+        if (prevRow != -1) listener->closeSheetRow();
+        int numRepeat;
+        double h=sheet.getRowHeight(prevRow+1, numRepeat);
+        if (cell.position()[1]<prevRow+1+numRepeat)
+          numRepeat=cell.position()[1]-1-prevRow;
+        listener->openSheetRow(float(h), librevenge::RVNG_POINT, numRepeat);
+        prevRow+=numRepeat;
       }
     }
+    if (cell.position()[1] > prevRow) {
+      if (prevRow != -1) listener->closeSheetRow();
+      listener->openSheetRow(float(sheet.getRowHeight(++prevRow)), librevenge::RVNG_POINT);
+    }
 
     GreatWksSSParserInternal::Style style=m_state->getStyle(cell.m_style);
     cell.setFont(style.m_font);
@@ -767,7 +798,7 @@ bool GreatWksSSParser::sendSpreadsheet()
       listener->setFont(style.m_font);
       input->seek(cell.m_content.m_textEntry.begin(), librevenge::RVNG_SEEK_SET);
       while (!input->isEnd() && input->tell()<cell.m_content.m_textEntry.end()) {
-        unsigned char c=(unsigned char) input->readULong(1);
+        unsigned char c=static_cast<unsigned char>(input->readULong(1));
         if (c==0xd)
           listener->insertEOL();
         else
@@ -791,7 +822,7 @@ bool GreatWksSSParser::readChart()
   libmwaw::DebugStream f;
 
   f << "Entries(Chart):";
-  long sz=(long) input->readULong(4);
+  long sz=long(input->readULong(4));
   long endPos=pos+4+sz;
   if (sz < 4 || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("GreatWksSSParser::readChart: can not find the chart zone size\n"));
@@ -800,8 +831,8 @@ bool GreatWksSSParser::readChart()
     ascii().addNote(f.str().c_str());
     return false;
   }
-  int N=(int) input->readULong(2);
-  int fSz=(int) input->readULong(2);
+  int N=static_cast<int>(input->readULong(2));
+  int fSz=static_cast<int>(input->readULong(2));
   if (N) f << "N=" << N << ",";
   if ((N && fSz!=0x14) || long(4+N*fSz)!=sz) {
     MWAW_DEBUG_MSG(("GreatWksSSParser::readChart: can not find the number of chart\n"));
@@ -837,7 +868,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   libmwaw::DebugStream f;
 
   f << "Entries(Cell):";
-  long sz=(long) input->readULong(4);
+  long sz=long(input->readULong(4));
   long endPos=pos+4+sz;
   if (sz < 0x12 || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("GreatWksSSParser::readCell: can not find a cell\n"));
@@ -848,13 +879,13 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   }
   int cPos[2];
   for (int i=0; i<2; ++i)
-    cPos[i]=(int) input->readLong(2);
-  Vec2i cellPos(cPos[0]-1,cPos[1]-1);
+    cPos[i]=static_cast<int>(input->readLong(2));
+  MWAWVec2i cellPos(cPos[0]-1,cPos[1]-1);
   cell.setPosition(cellPos);
   f << "cell=" << cellPos << ",";
-  cell.m_style=(int) input->readLong(2);
+  cell.m_style=static_cast<int>(input->readLong(2));
   f << "style" << cell.m_style << ",";
-  int val=(int) input->readULong(2);
+  int val=static_cast<int>(input->readULong(2));
   int form=(val>>8)&0x1f;
   MWAWCell::Format format;
   format.m_format=MWAWCell::F_TEXT;
@@ -927,7 +958,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   val &=0x20F0;
   if (val)
     f << "fl0=" << std::hex << val << std::dec << ",";
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val&0xf) {
     f << "bord=";
     int wh=0;
@@ -978,10 +1009,10 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   val &= 0x78F0;
   if (val)
     f << "fl1=" << std::hex << val << std::dec << ",";
-  val=(int) input->readULong(2); // small number between 0 and 255
+  val=static_cast<int>(input->readULong(2)); // small number between 0 and 255
   if (val)
     f << "f0=" << val << ",";
-  int formSz=(int) input->readULong(2);
+  int formSz=static_cast<int>(input->readULong(2));
   if (formSz&0x8000) { // appear if now, rand, ... is used
     f << "formula[update],";
     formSz &= 0x7FFF;
@@ -1016,7 +1047,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   }
 
   if (ok && input->tell()+4<=endPos) {
-    int typeVal=(int) input->readLong(2);
+    int typeVal=static_cast<int>(input->readLong(2));
     long valuePos=input->tell();
     switch (typeVal) {
     case 1: // no value
@@ -1048,7 +1079,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
     case 7: {
       if (content.m_contentType!=MWAWCellContent::C_FORMULA)
         content.m_contentType=MWAWCellContent::C_TEXT;
-      int sSz=(int) input->readULong(1);
+      int sSz=static_cast<int>(input->readULong(1));
       if (valuePos+1+sSz+2>endPos) {
         MWAW_DEBUG_MSG(("GreatWksSSParser::readCell: can not read a string value\n"));
         f << "###";
@@ -1059,7 +1090,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
       content.m_textEntry.setLength(sSz);
       std::string text("");
       for (int i=0; i<sSz; ++i)
-        text += (char) input->readULong(1);
+        text += char(input->readULong(1));
       f << "\"" << text << "\",";
       if ((sSz%2)==0)
         input->seek(1, librevenge::RVNG_SEEK_CUR);
@@ -1108,7 +1139,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
     }
   }
   if (ok && input->tell()+2<=endPos) {
-    val =(int) input->readULong(2);
+    val =static_cast<int>(input->readULong(2));
     if (!input->checkPosition(input->tell()+4*val)) {
       MWAW_DEBUG_MSG(("GreatWksSSParser::readCell: can not read the number of childs\n"));
       f << "###";
@@ -1141,7 +1172,7 @@ bool GreatWksSSParser::readStyles()
   libmwaw::DebugStream f;
 
   f << "Entries(Style):";
-  long sz=(long) input->readULong(4);
+  long sz=long(input->readULong(4));
   long endPos=pos+4+sz;
   int expectedSize=vers==1 ? 18 : 40;
   if (!input->checkPosition(endPos) || (sz%expectedSize)) {
@@ -1160,13 +1191,13 @@ bool GreatWksSSParser::readStyles()
     f << "Style-" << i << ":";
     GreatWksSSParserInternal::Style style;
     MWAWFont &font=style.m_font;
-    int val=(int) input->readLong(2); // always 0 ?
+    int val=static_cast<int>(input->readLong(2)); // always 0 ?
     if (val) f << "#unkn=" << val << ",";
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val!=1) f << "used?=" << val << ",";
 
-    font.setId(m_document->getTextParser()->getFontId((int) input->readULong(2)));
-    int flag =(int) input->readULong(2);
+    font.setId(m_document->getTextParser()->getFontId(static_cast<int>(input->readULong(2))));
+    int flag =static_cast<int>(input->readULong(2));
     uint32_t flags=0;
     if (flag&0x1) flags |= MWAWFont::boldBit;
     if (flag&0x2) flags |= MWAWFont::italicBit;
@@ -1185,10 +1216,10 @@ bool GreatWksSSParser::readStyles()
     flag &=0xD480;
     if (flag) f << "#fl=" << std::hex << flag << std::dec << ",";
     font.setFlags(flags);
-    font.setSize((float) input->readULong(2));
+    font.setSize(float(input->readULong(2)));
     unsigned char color[3];
     for (int c=0; c<3; ++c)
-      color[c] = (unsigned char)(input->readULong(2)>>8);
+      color[c] = static_cast<unsigned char>(input->readULong(2)>>8);
     font.setColor(MWAWColor(color[0],color[1],color[2]));
     f << font.getDebugString(getParserState()->m_fontConverter) << ",";
     f << "h[line]?=" << input->readULong(2) << ",";
@@ -1201,14 +1232,14 @@ bool GreatWksSSParser::readStyles()
     MWAWColor bfColors[2];
     for (int j=0; j<2; ++j) {  // front/back color?
       for (int c=0; c<3; ++c)
-        color[c] = (unsigned char)(input->readULong(2)>>8);
+        color[c] = static_cast<unsigned char>(input->readULong(2)>>8);
       MWAWColor col(color[0],color[1],color[2]);
       bfColors[j]=col;
       if ((j==0 && col.isBlack()) || (j==1 && col.isWhite())) continue;
       if (j==0) f << "col[front]=" << MWAWColor(color[0],color[1],color[2]) << ",";
       else f << "col[back]=" << MWAWColor(color[0],color[1],color[2]) << ",";
     }
-    int patId=(int) input->readLong(2);
+    int patId=static_cast<int>(input->readLong(2));
     if (!patId) {
       style.m_backgroundColor=bfColors[1];
       input->seek(8, librevenge::RVNG_SEEK_CUR);
@@ -1216,10 +1247,10 @@ bool GreatWksSSParser::readStyles()
     else {
       f << "pattern[id]=" << patId << ",";
       MWAWGraphicStyle::Pattern pattern;
-      pattern.m_dim=Vec2i(8,8);
+      pattern.m_dim=MWAWVec2i(8,8);
       pattern.m_data.resize(8);
       for (size_t j=0; j < 8; ++j)
-        pattern.m_data[j]=(unsigned char) input->readULong(1);
+        pattern.m_data[j]=static_cast<unsigned char>(input->readULong(1));
       pattern.m_colors[0]=bfColors[1];
       pattern.m_colors[1]=bfColors[0];
       pattern.getAverageColor(style.m_backgroundColor);
