@@ -105,7 +105,9 @@ std::ostream &operator<<(std::ostream &o, Frame const &ft)
     o<< "object,";
     break;
   case Frame::Unknown:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
 
@@ -262,8 +264,8 @@ MWAWTextListenerPtr MsWks4Zone::createListener(librevenge::RVNGTextInterface *in
   pagesH.resize(size_t(m_state->m_numPages), int(72.*getTextHeight()));
   m_document->getGraphParser()->computePositions(-1, linesH, pagesH);
   m_document->getGraphParser()->setPageLeftTop
-  (Vec2f(72.f*float(m_parserState->m_pageSpan.getMarginLeft()),
-         72.f*float(m_parserState->m_pageSpan.getMarginTop())+float(m_state->m_headerHeight)));
+  (MWAWVec2f(72.f*float(m_parserState->m_pageSpan.getMarginLeft()),
+             72.f*float(m_parserState->m_pageSpan.getMarginTop())+float(m_state->m_headerHeight)));
 
   return res;
 }
@@ -307,9 +309,9 @@ bool MsWks4Zone::parseHeaderIndexEntry(MWAWInputStreamPtr &input)
     name.append(1, char(input->readULong(1)));
 
     if (name[i] != 0 && name[i] != 0x20 &&
-        (41 > (uint8_t)name[i] || (uint8_t)name[i] > 90)) {
+        (41 > static_cast<uint8_t>(name[i]) || static_cast<uint8_t>(name[i]) > 90)) {
       MWAW_DEBUG_MSG(("MsWks4Zone:parseHeaderIndexEntry: bad character=%u (0x%02x) in name in header index\n",
-                      (unsigned int)name[i], (unsigned int)name[i]));
+                      static_cast<unsigned int>(name[i]), static_cast<unsigned int>(name[i])));
       m_document->ascii().addNote("###IndexEntry bad name(ignored)");
 
       input->seek(pos + cch, librevenge::RVNG_SEEK_SET);
@@ -319,24 +321,24 @@ bool MsWks4Zone::parseHeaderIndexEntry(MWAWInputStreamPtr &input)
 
   f << "Entries("<<name<<")";
   if (cch != 24) f << ", ###size=" << int(cch);
-  int id = (int) input->readULong(2);
+  int id = static_cast<int>(input->readULong(2));
   f << ", id=" << id << ", (";
   for (int i = 0; i < 2; i ++) {
-    int val = (int) input->readLong(2);
+    int val = static_cast<int>(input->readLong(2));
     f << val << ",";
   }
 
   std::string name2;
   for (size_t i =0; i < 4; i++)
-    name2.append(1, (char) input->readULong(1));
+    name2.append(1, char(input->readULong(1)));
   f << "), " << name2;
 
   MWAWEntry hie;
   hie.setName(name);
   hie.setType(name2);
   hie.setId(id);
-  hie.setBegin((long) input->readULong(4));
-  hie.setLength((long) input->readULong(4));
+  hie.setBegin(long(input->readULong(4)));
+  hie.setLength(long(input->readULong(4)));
 
   f << ", offset=" << std::hex << hie.begin() << ", length=" << hie.length();
 
@@ -379,9 +381,9 @@ bool MsWks4Zone::parseHeaderIndex(MWAWInputStreamPtr &input)
   input->seek(0x08, librevenge::RVNG_SEEK_SET);
 
   long pos = input->tell();
-  int i0 = (int) input->readLong(2);
-  int i1 = (int) input->readLong(2);
-  uint16_t n_entries = (uint16_t) input->readULong(2);
+  int i0 = static_cast<int>(input->readLong(2));
+  int i1 = static_cast<int>(input->readLong(2));
+  uint16_t n_entries = static_cast<uint16_t>(input->readULong(2));
   // fixme: sanity check n_entries
 
   libmwaw::DebugStream f;
@@ -402,9 +404,9 @@ bool MsWks4Zone::parseHeaderIndex(MWAWInputStreamPtr &input)
 
     pos = input->tell();
     f.str("");
-    uint16_t unknown1 = (uint16_t) input->readULong(2); // function of the size of the entries ?
+    uint16_t unknown1 = static_cast<uint16_t>(input->readULong(2)); // function of the size of the entries ?
 
-    uint16_t n_entries_local = (uint16_t) input->readULong(2);
+    uint16_t n_entries_local = static_cast<uint16_t>(input->readULong(2));
     f << "Header("<<std::hex << unknown1<<"): N=" << std::dec << n_entries_local;
 
     if (n_entries_local > 0x20) {
@@ -412,7 +414,7 @@ bool MsWks4Zone::parseHeaderIndex(MWAWInputStreamPtr &input)
       return readSome;
     }
 
-    uint32_t next_index_table = (uint32_t) input->readULong(4);
+    uint32_t next_index_table = static_cast<uint32_t>(input->readULong(4));
     f << std::hex << ", nextHeader=" << next_index_table;
     if (next_index_table != 0xFFFFFFFF && long(next_index_table) < pos) {
       MWAW_DEBUG_MSG(("MsWks4Zone::parseHeaderIndex: error: next_index_table=%x decreasing !!!\n", next_index_table));
@@ -630,9 +632,9 @@ bool MsWks4Zone::readPRNT(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAW
     return false;
   }
   else {
-    Vec2i paperSize = info.paper().size();
-    Vec2i pageSize = info.page().size();
-    Vec2i margin = paperSize - pageSize;
+    MWAWVec2i paperSize = info.paper().size();
+    MWAWVec2i pageSize = info.page().size();
+    MWAWVec2i margin = paperSize - pageSize;
     margin.set(margin.x()/2, margin.y()/2);
 
     page.setMarginTop(margin.y()/72.0);
@@ -677,14 +679,14 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
   long endPage = entry.end();
 
   input->seek(debPage, librevenge::RVNG_SEEK_SET);
-  int sz = (int) input->readULong(1);
+  int sz = static_cast<int>(input->readULong(1));
   if (sz != length-1) f << "###sz=" << sz << ",";
 
   bool ok = true;
   double dim[6] = {-1., -1., -1., -1., -1., -1. };
   while (input->tell() < endPage) {
     long debPos = input->tell();
-    int val = (int) input->readULong(1);
+    int val = static_cast<int>(input->readULong(1));
 
     switch (val) {
     case 0x41: // w (or 4000 when frame)
@@ -697,7 +699,7 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
         ok = false;
         break;
       }
-      long d = (int) input->readULong(2);
+      long d = static_cast<int>(input->readULong(2));
       if (d == 4000) break;
       dim[val-0x41] = double(d)/72.;
       break;
@@ -708,7 +710,7 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
         ok = false;
         break;
       }
-      int v = (int) input->readULong(2);
+      int v = static_cast<int>(input->readULong(2));
       m_state->m_numColumns=v+1;
       if (v) f2 << "cols=" << m_state->m_numColumns << ",";
       break;
@@ -718,7 +720,7 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
         ok = false;
         break;
       }
-      int v = (int) input->readLong(1);
+      int v = static_cast<int>(input->readLong(1));
       if (v==1) {
         m_state->m_hasColumnSep=true;
         f2 << "hasColSep,";
@@ -734,7 +736,7 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
         ok = false;
         break;
       }
-      int v = (int) input->readULong(2);
+      int v = static_cast<int>(input->readULong(2));
       f2 << "f" << val;
       if (v != 1) f2 << "=" << v << "#";
       f2 << ",";
@@ -746,7 +748,7 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
         ok = false;
         break;
       }
-      int v = (int) input->readULong(2);
+      int v = static_cast<int>(input->readULong(2));
       f2 << "f" << val << "=" << std::hex << v << std::dec << ",";
       break;
     }
@@ -770,7 +772,7 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
         ok = false;
         break;
       }
-      int v = (int) input->readLong(2);
+      int v = static_cast<int>(input->readLong(2));
       f2 << "f" << val << "=" <<  v << ",";
       break;
     }
@@ -780,7 +782,7 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
         ok = false;
         break;
       }
-      int v = (int) input->readLong(1);
+      int v = static_cast<int>(input->readLong(1));
       f2 << "f" << val << "=" <<  v << ",";
       break;
     }
@@ -794,7 +796,7 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
         ok = false;
         break;
       }
-      int v = (int) input->readLong(1);
+      int v = static_cast<int>(input->readLong(1));
       f2 << "f" << val << "=" <<  v << ",";
       break;
     }
@@ -917,7 +919,7 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
   long endPage = entry.end();
 
   input->seek(debPage, librevenge::RVNG_SEEK_SET);
-  int numFram = (int) input->readULong(2);
+  int numFram = static_cast<int>(input->readULong(2));
   if (numFram <= 0) return false;
   entry.setParsed(true);
 
@@ -928,7 +930,7 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
 
   for (int i = 0; i < numFram; i++) {
     long debPos = input->tell();
-    int size = (int) input->readULong(1);
+    int size = static_cast<int>(input->readULong(1));
     if (size <= 0) break;
 
     f.str("");
@@ -938,9 +940,9 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
     if (endPos > endPage) break;
 
     MsWks4ZoneInternal::Frame frame;
-    Vec2f fOrig, fSiz;
+    MWAWVec2f fOrig, fSiz;
     while (input->tell() < endPos) {
-      int val = (int) input->readULong(1);
+      int val = static_cast<int>(input->readULong(1));
       long pos = input->tell();
 
       int sz = 0;
@@ -969,7 +971,7 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
           ok = false;
           break;
         }
-        int value = (int) input->readLong(2);
+        int value = static_cast<int>(input->readLong(2));
         switch (value) {
         case 1:
           frame.m_type = MsWks4ZoneInternal::Frame::Textbox;
@@ -990,7 +992,7 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
           ok = false;
           break;
         }
-        int value = (int) input->readLong(2);
+        int value = static_cast<int>(input->readLong(2));
         if (value != -1) f << "###29=" << value << ",";
       }
       break;
@@ -1025,12 +1027,12 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
         }
         std::string fName("");
         for (int j = 0; j < 4; j++) {
-          char c = (char) input->readULong(1);
+          char c = char(input->readULong(1));
           if (c == '\0') continue;
           fName +=  c;
         }
         frame.m_pictId.setName(fName);
-        frame.m_pictId.setId((int) input->readULong(2));
+        frame.m_pictId.setId(static_cast<int>(input->readULong(2)));
         done = true;
       }
       break;
@@ -1050,7 +1052,7 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
 
       f << std::hex << val << "=" << std::dec;
       if (sz) {
-        int v2 = (int) input->readLong(sz);
+        int v2 = static_cast<int>(input->readLong(sz));
         f << v2;
       }
       f << ", ";
@@ -1091,7 +1093,7 @@ bool MsWks4Zone::readSELN(MWAWInputStreamPtr input, MWAWEntry const &entry)
   }
   entry.setParsed(true);
 
-  int type = (int) input->readLong(1);
+  int type = static_cast<int>(input->readLong(1));
   // 2,3
   switch (type) {
   case 2:
@@ -1106,7 +1108,7 @@ bool MsWks4Zone::readSELN(MWAWInputStreamPtr input, MWAWEntry const &entry)
   }
   // 0 0xFF 0
   for (int i = 0; i < 3; i++) {
-    int val = (int)  input->readLong(1);
+    int val = static_cast<int>(input->readLong(1));
     if ((i%2)+val)
       f << "unk" << i << "=" << val << ",";
   }
@@ -1121,7 +1123,7 @@ bool MsWks4Zone::readSELN(MWAWInputStreamPtr input, MWAWEntry const &entry)
   // I find f2=-1,f20=-1,f21=-1 and f22=0|1
   // maybe f22=0 -> mainTextZone, f22=1 -> header, ...
   for (int i = 0; i < (endPage-debPage-12)/2; i++) {
-    int val = (int) input->readLong(2);
+    int val = static_cast<int>(input->readLong(2));
     if (val) f << ",f" << i << "=" << val;
   }
 
